@@ -169,6 +169,9 @@
 #' \code{\link{summarizeEffects}}, to summarize the pairwise effects returned when \code{all.pairwise=TRUE}.
 #'
 #' \code{\link{reportGroupMarkerStatistics}}, to consolidate the statistics for a single group into its own data frame.
+#'
+#' \code{\link{scoreMarkers.se}}, to score markers from a \link[SummarizedExperiment]{SummarizedExperiment}.
+#'
 #' @export
 #' @importFrom beachmat initializeCpp
 scoreMarkers <- function(
@@ -253,16 +256,7 @@ scoreMarkers <- function(
         for (nm in setdiff(names(output), c("mean", "detected"))) {
             current <- output[[nm]]
             names(current) <- groups$names
-            for (i in seq_along(current)) {
-                if (length(current[[i]])) {
-                    df <- data.frame(current[[i]])
-                } else {
-                    df <- data.frame(matrix(0, ngenes, 0))
-                }
-                rownames(df) <- rn
-                current[[i]] <- df
-            }
-            output[[nm]] <- current
+            output[[nm]] <- format_summary_output(current, ngenes, rn, compute.summary.quantiles)
         }
 
     } else {
