@@ -1,8 +1,7 @@
-//#include "config.h"
+#include "config.h"
 
 #include "clrm1/clrm1.hpp"
-#include "Rcpp.h"
-#include "Rtatami.h"
+#include "sanisizer/sanisizer.hpp"
 
 //[[Rcpp::export(rng=false)]]
 Rcpp::NumericVector compute_clrm1_factors(SEXP x, int num_threads) {
@@ -11,7 +10,7 @@ Rcpp::NumericVector compute_clrm1_factors(SEXP x, int num_threads) {
 
     clrm1::Options opt;
     opt.num_threads = num_threads;
-    Rcpp::NumericVector output(mat->ncol());
+    auto output = sanisizer::create<Rcpp::NumericVector>(mat->ncol());
     clrm1::compute(*mat, opt, static_cast<double*>(output.begin()));
     return output;
 }
